<?php

declare(strict_types=1);

namespace hood\vk\robots;

use hood\vk\robots\robot,
    hood\vk\api\longpoll;

use Throwable,
    Exception;

/**
 * Робот-группа
 *
 * @property longpoll $longpoll LongPoll-сессия
 *
 * @method public function __set($name, $value) Запись свойства
 * @method public function __get($name) Чтение свойства
 * @method public function __isset($name) Проверка на инициализированность свойства
 *
 * @package hood\vk\robots
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 */
final class group extends robot
{
    /**
     * @var longpoll $longpoll LongPoll-сессия
     */
    protected longpoll $longpoll;

    /**
     * Запись свойства
     *
     * @param string $name Название
     * @param mixed $value Значение
     *
     * @see hood\vk\robots\robot Наследуемый метод
     *
     * @return void
     */
    public function __set(string $name, $value): void
    {
        try {
            parent::__set($name, $value);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            match ($name) {
                'longpoll' => match (true) {
                    $value instanceof LongPoll => $this->longpoll = $value,
                    default => $this->longpoll = new LongPoll($this)
                }
            };

            throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious());
        }
    }

    /**
     * Чтение свойства
     *
     * @param string $name Название
     *
     * @see hood\vk\robots\robot Наследуемый метод
     *
     * @return mixed
     */
    public function __get(string $name)
    {
        try {
            return parent::__get($name);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            if ($name === 'longpoll') {
                if (!isset($this->longpoll)) {
                    $this->longpoll = new LongPoll($this);
                }
                return $this->longpoll;
            }

            throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious());
        }
    }

    /**
     * Проверка на инициализированность свойства
     *
     * @param string $name Название
     *
     * @return mixed
     */
    public function __isset(string $name)
    {
        try {
            return parent::__isset($name);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            if ($name === 'longpoll') {
                return isset($this->longpoll);
            }

            throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious());
        }
    }
}
