<?php

declare(strict_types=1);

use PHPUnit\Framework\TestCase;

use hood\vk\core,
    hood\vk\robots\robot,
    hood\vk\tests\settings;

use hood\accounts\vk as account;

/**
 * @testdox Робот-группа
 */
final class messagesTest extends TestCase
{
    use settings;

    /**
     * @var core $core Ядро фреймворка
     */
    private static core $core;

    /**
     * @var account $account Аккаунт
     */
    protected static account $account;

    /**
     * @var robot $robot Робот
     */
    private static robot $robot;


    /**
     * @testdox Деинициализация аккаунта
     * @afterClass
     */
    public static function testAccountDeinit(): void
    {
        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, __DIR__ . '../../accounts');

        // Деаутентификация
        self::$account->deauth();
    }

    /**
     * @testdox Инициализация аккаунта
     * @beforeClass
     */
    public function testAccountInit(): void
    {
        // Проверка входных данных
        if (empty(self::$project_id)) {
            self::$markTestSkipped('Не найден идентификатор проекта');
        }

        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, __DIR__ . '../../accounts');

        // Запись режима SSL-протокола
        self::$account->ssl = self::$ssl ?? true;

        if (empty(self::$key)) {
            // Если не указан ключ

            // Проверка входных данных
            if (empty(self::$login)) {
                self::$markTestSkipped('Не найден входной аккаунта');
            } else if (empty(self::$password)) {
                self::$markTestSkipped('Не найден пароль аккаунта');
            }

            // Деаутентификация (на всякий случай)
            self::$account->deauth();

            // Аутентификация и генерация ключа
            self::$key = self::$account->auth(self::$login, self::$password)->key(self::$project_id);
        }

        $this->assertNotNull(self::$key, 'Ошибка при инициализации ключа аккаунта');
    }

    /**
     * @testdox Деинициализация робота
     * @after
     */
    public function testRobotGroupDeinit(): void
    {
        // Очистка реестра
        self::$core->delete();

        // Проверка
        $this->assertEmpty(self::$core->get(self::$robot->id), 'Ошибка при деинициализации робота');
    }

    /**
     * @testdox Инициализация робота
     * @before
     */
    public function testRobotGroupInit(): void
    {
        // Инициализация ядра
        self::$core = core::init();

        // Сохранение количества роботов
        $count = self::$core->robots;

        // Инициализация робота
        self::$robot = self::$core->group(empty(self::$group_id) ? rand(0, 10) : self::$group_id);

        // Проверка
        $this->assertEquals(self::$core->robots, $count + 1, 'Ошибка при инициализации робота');
    }


    /**
     * @testdox Инициализация робота
     */
    public function testApiMethodMessagesNew(): void
    {
        var_dump(self::$robot->message('Тестирование отправки сообщения', self::$target_id ?? 'pizba228'));
    }
}
